﻿using System;
using System.Text.RegularExpressions;
using MbnApi;

namespace Aero2AutoReconnect
{
    internal class Aero2 : IMbnConnectionEvents
    {
        private string profileName = null, interfaceID = null;
        private uint eventCookie;

        public event Action Disconnected;
        public event Action Connected;

        public void Check()
        {
            IMbnConnection connection = getConnection();
            checkConnection(connection);
          }

        public void Start()
        {
            Stop();
            IMbnConnection connection = getConnection();
            checkConnection(connection);

            var connectionManager = (IMbnConnectionManager)new MbnConnectionManager();
            var connectionPointContainer = (IConnectionPointContainer)connectionManager;
            var guid = typeof(IMbnConnectionEvents).GUID;
            IConnectionPoint connectionPoint;
            connectionPointContainer.FindConnectionPoint(ref guid, out connectionPoint);
            connectionPoint.Advise(this, out eventCookie);
        }

        private IMbnConnection getConnection()
        {
            var interfaceManager = (IMbnInterfaceManager)new MbnInterfaceManager();
            var arrIntefaces = (IMbnInterface[])interfaceManager.GetInterfaces();
            if (arrIntefaces == null || arrIntefaces.Length == 0)
                throw new ApplicationException("Mobile broadband interface was not found.");
            var connectionProfileManager = (IMbnConnectionProfileManager)new MbnConnectionProfileManager();
            IMbnConnection connection = null;
            for (var i = 0; i < arrIntefaces.Length; i++)
            {
                var mbnInterface = arrIntefaces[i];
                var connectionProfiles = connectionProfileManager.GetConnectionProfiles(mbnInterface) as IMbnConnectionProfile[];
                if (connectionProfiles == null || connectionProfiles.Length == 0)
                    continue;
                foreach (var profile in connectionProfiles)
                {
                    var xml = profile.GetProfileXmlData();
                    if (xml.Contains(@"<AccessString>darmowy</AccessString>") && xml.Contains(@"<IsDefault>true</IsDefault>"))
                    {
                        var match = Regex.Match(xml, @"<Name>([^<]+)</Name>");
                        if (match.Success)
                        {
                            profileName = match.Groups[1].Value;
                            connection = mbnInterface.GetConnection();
                            this.interfaceID = mbnInterface.InterfaceID;
                            break;
                        }
                    }
                }
            }
            if (interfaceID == null || connection == null)
                throw new ApplicationException("Aero2 profile was not found.");
            return connection;
        }

        public void Stop()
        {
            if (eventCookie != 0)
            {
                var connectionManager = (IMbnConnectionManager)new MbnConnectionManager();
                var connectionPointContainer = (IConnectionPointContainer)connectionManager;
                var guid = typeof(IMbnConnectionEvents).GUID;
                IConnectionPoint connectionPoint;
                connectionPointContainer.FindConnectionPoint(ref guid, out connectionPoint);
                connectionPoint.Unadvise(eventCookie);
                profileName = null;
                interfaceID = null;
                eventCookie = 0;
            }
        }

        public void OnConnectComplete(IMbnConnection newConnection, uint requestID, int status)
        {
            if (newConnection.InterfaceID == this.interfaceID && status != 0)
            {
                if (Disconnected != null)
                    Disconnected();
                newConnection.Connect(MBN_CONNECTION_MODE.MBN_CONNECTION_MODE_PROFILE, profileName, out requestID);
            }
            else
            {
                if (Connected != null)
                    Connected();
            }
        }

        public void OnConnectStateChange(IMbnConnection newConnection)
        {
            if (newConnection.InterfaceID != this.interfaceID)
                return;
            checkConnection(newConnection);
        }

        private void checkConnection(IMbnConnection connection)
        {
            MBN_ACTIVATION_STATE connectonState;
            string tempProfileName;
            uint requestID;
            connection.GetConnectionState(out connectonState, out tempProfileName);

            if (connectonState == MBN_ACTIVATION_STATE.MBN_ACTIVATION_STATE_ACTIVATED)
            {
                if (Connected != null)
                    Connected();
            }
            else
            {
                if (Disconnected != null)
                    Disconnected();
            }

            if (connectonState == MBN_ACTIVATION_STATE.MBN_ACTIVATION_STATE_DEACTIVATED)
            {
                connection.Connect(MBN_CONNECTION_MODE.MBN_CONNECTION_MODE_PROFILE, profileName, out requestID);
            }
        }

        public void OnDisconnectComplete(IMbnConnection newConnection, uint reqeustID, int status)
        {
            if (Disconnected != null)
                Disconnected();
        }

        public void OnVoiceCallStateChange(IMbnConnection newConnection)
        {
        }
    }
}
