﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Aero2AutoReconnect
{
    public class Aero2Form : Form
    {
        Aero2 aero2 = new Aero2();
        Timer timer = new Timer();
        NotifyIcon notifyIcon = new NotifyIcon();
        DateTime beginConnectionDateTime = DateTime.MinValue;

        public Aero2Form()
        {
            notifyIcon.Icon = Icons.dc;
            notifyIcon.Visible = true;

            aero2.Connected += new Action(aero2_Connected);
            aero2.Disconnected += new Action(aero2_Disconnected);

            this.FormClosed += new FormClosedEventHandler(Aero2Form_FormClosed);

            timer_Tick(null, null);

            timer.Interval = 1000 * 60;
            timer.Enabled = true;
            timer.Tick += new EventHandler(timer_Tick);
            timer.Start();
        }

        void aero2_Connected()
        {
            if (ignoreConnect)
                return;
            if (this.InvokeRequired)
                this.Invoke(new Action(aero2_Connected));
            else
            {
                beginConnectionDateTime = DateTime.Now;
                setNotifyIconAndText();
            }
        }

        void aero2_Disconnected()
        {
            if (this.InvokeRequired)
                this.Invoke(new Action(aero2_Connected));
            else
            {
                beginConnectionDateTime = DateTime.MinValue;
                notifyIcon.Icon = Icons.dc;
                setNotifyIconAndText();
            }
        }

        private void setNotifyIconAndText()
        {
            if (beginConnectionDateTime != DateTime.MinValue)
            {
                notifyIcon.Text = null;
                var connectionTimeInMinutes = (DateTime.Now - beginConnectionDateTime).TotalMinutes;
                if (connectionTimeInMinutes > 55)
                    notifyIcon.Icon = Icons.n1;
                else if (connectionTimeInMinutes > 45)
                    notifyIcon.Icon = Icons.n2;
                else if (connectionTimeInMinutes > 20)
                    notifyIcon.Icon = Icons.n3;
                else
                    notifyIcon.Icon = Icons.n4;
                notifyIcon.Text = System.Threading.Thread.CurrentThread.CurrentCulture.TwoLetterISOLanguageName == "pl" ? "Pozostało minut: " : "Minutes remaining: " + (60 - Math.Round(connectionTimeInMinutes, MidpointRounding.AwayFromZero));
            }
        }

        private bool stated = false;
        bool ignoreConnect = false;
        void timer_Tick(object sender, EventArgs e)
        {
            try
            {
                if (!stated)
                {
                    aero2.Start();
                    stated = true;
                }
                else
                {
                    ignoreConnect = true;
                    aero2.Check();
                    ignoreConnect = false;
                    setNotifyIconAndText();
                }
            }
            catch (Exception ex)
            {
                beginConnectionDateTime = DateTime.MinValue;
                notifyIcon.Icon = Icons.er;
                notifyIcon.Text = ex.Message;
            }
        }

        void Aero2Form_FormClosed(object sender, FormClosedEventArgs e)
        {
            notifyIcon.Visible = false;
            aero2.Stop();
        }
    }
}
